/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;
import javax.sql.DataSource;

import net.smartlab.web.BusinessObjectFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dbunit.DatabaseTestCase;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.DefaultDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.hibernate.cfg.Configuration;
import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.DialectFactory;
import org.mockejb.jndi.MockContextFactory;

import com.mchange.v2.c3p0.DataSources;

/**
 * Base class for BusinessObjectFactory testing.
 * 
 * @author rlogiacco
 */
public abstract class BusinessObjectFactoryTestCase extends DatabaseTestCase {

	/**
	 * TODO documentation
	 */
	protected static final Map factories = new HashMap();

	/**
	 * Provides logging for the test cases.
	 */
	protected Log logger = LogFactory.getLog(this.getClass());

	/**
	 * The hibernate dialect stores informations useful for dbunit customization
	 */
	private Dialect dialect;
	
	/**
	 * The datasource providing connections both to hibernate and dbunit
	 */
	private DataSource datasource;
	
	/**
	 * Default constructor.
	 */
	public BusinessObjectFactoryTestCase() {
		super();
	}

	/**
	 * Named constructor.
	 * 
	 * @param name test name.
	 */
	public BusinessObjectFactoryTestCase(String name) {
		super(name);
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("setUp() - start");
		}
		MockContextFactory.setAsInitial();
		BusinessObjectFactoryTestSupport support = new BusinessObjectFactoryTestSupport(this.getFactoryConfiguration());
		BusinessObjectFactory.setConfigurationStrategy(support);

		InitialContext context = new InitialContext();
		try {
			datasource = (DataSource)context.lookup(this.getClass().getCanonicalName() + "/ds");
			dialect = (Dialect)context.lookup(this.getClass().getCanonicalName() + "/hd");
		} catch (NameNotFoundException nnfe) {
			Configuration configuration = new Configuration().configure(getFactoryConfiguration());
			try {
				Properties props = new Properties();
				props.load(Object.class.getResourceAsStream("/hibernate.properties"));
				configuration.addProperties(props);
			} catch (FileNotFoundException fne) {
				logger.warn("hibernate.properties file not found");
			}
			dialect = DialectFactory.buildDialect(configuration.getProperty("hibernate.dialect"));
			Class.forName(configuration.getProperty("hibernate.connection.driver_class"));
			datasource = DataSources.unpooledDataSource(
					configuration.getProperty("hibernate.connection.url"), 
					configuration.getProperty("hibernate.connection.username"), 
					configuration.getProperty("hibernate.connection.password"));
			if (configuration.getProperty("connection.datasource") != null) {
				context.bind(configuration.getProperty("connection.datasource"), datasource);
			}
			context.bind(this.getClass().getCanonicalName() + "/ds", datasource);
			context.bind(this.getClass().getCanonicalName() + "/hd", dialect);
		}		
		super.setUp();
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("tearDown() - start");
		}
		BusinessObjectFactory.flush();
		super.tearDown();
	}

	/**
	 * Returns the configuration to be used for test purposes. The default
	 * implementation uses the file <code>res/smartweb.jar.hcf</code> but the
	 * method could be overriden to support different configuration files for
	 * different tests.
	 * 
	 * @return the configuration to be used for test purposes.
	 * @throws Exception if the resource fails to open.
	 */
	protected URL getFactoryConfiguration() throws Exception {
		return new File("res/smartweb.jar.hcf").toURI().toURL();
	}

	/**
	 * @deprecated renamed to getFactoryConfiguration
	 * @see getFactoryConfiguration()
	 */
	protected URL getConfigurationFile() throws Exception {
		return this.getFactoryConfiguration();
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getConnection()
	 * @uml.property name="connection"
	 */
	protected IDatabaseConnection getConnection() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getConnection() - start");
		}
		if (datasource == null || dialect == null) {
			this.setUp();
		}
		DatabaseConnection connection = new DatabaseConnection(datasource.getConnection());
		connection.getConfig().setFeature(DatabaseConfig.FEATURE_QUALIFIED_TABLE_NAMES, dialect.qualifyIndexName());
		connection.getConfig().setProperty(DatabaseConfig.PROPERTY_ESCAPE_PATTERN,
				dialect.openQuote() + "?" + dialect.closeQuote());
		return connection;
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getDataSet()
	 */
	protected IDataSet getDataSet() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getDataSet() - start");
			logger.trace("  res = " + this.getDataSetFile());
		}
		try {
			return new FlatXmlDataSet(new FileInputStream(this.getDataSetFile()));
		} catch (Exception e) {
			logger.warn("getDataSet() - not found");
			return new DefaultDataSet();
		}
	}

	/**
	 * Method implementations should return the dataset file name.
	 * 
	 * @return the dataset file name.
	 */
	protected abstract String getDataSetFile();

	/**
	 * Partial dataset export allows to select which tables to include into the
	 * dataset.
	 * 
	 * @param tables a two dimensions String array
	 * @throws Exception if something wrong occurs during dataset creation.
	 * @deprecated other dataset creation procedures are available, please see http://www.dbunit.org
	 */
	public void export(String[][] tables) throws Exception {
		// partial database export
		QueryDataSet dataSet = new QueryDataSet(this.getConnection());
		for (int i = 0; i < tables.length; i++) {
			if (tables[i][1] == null) {
				dataSet.addTable(tables[i][0]);
			} else {
				dataSet.addTable(tables[i][0], tables[i][1]);
			}
		}
		FlatXmlDataSet.write(dataSet, new FileOutputStream(this.getDataSetFile()));
	}
}
